/*****************************************************************************
 *   adc.c:  ADC module file for NXP LPC29xx Family Microprocessors
 *
 *   Copyright(C) 2007, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2007.08.15  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/
#include "LPC29xx.h"                        /* LPC29xx definitions */
#include "type.h"
#include "irq.h"
#include "target.h"
#include "adc.h"

volatile DWORD ADC2Value[ITERATION][ADC_NUM];
volatile DWORD ADC2ValueAverage[ADC_NUM];
volatile DWORD ADC2IntDone = 0;
volatile DWORD ADC2Iteration = 0;

/******************************************************************************
** Function name:		ADC2Handler
**
** Descriptions:		ADC2 interrupt handler
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
void ADC2_Handler (void)
{
  DWORD regVal;
  
  regVal = ADC2_INT_STATUS;
  if ( regVal & (0x01<<0) )	/* check EOS error first */
  {
	ADC2_Stop();
	if ( ADC2Iteration >= ITERATION )
	{
	  ADC2Iteration = 0;
	}
	ADC2Value[ADC2Iteration][0] = ADC2_ACD7 & 0x3FF;
	ADC2Iteration++;
	ADC2IntDone = 1;
	ADC2_INT_CLR_STATUS |= (0x01<<0);
  }
    
  if ( regVal & (0x01<<1) )	/* ADC2 Compare */
  {
	ADC2_INT_CLR_STATUS |= (0x01<<1);
  }
  return;
}

/*****************************************************************************
** Function name:		ADC2_Init
**
** Descriptions:		initialize ADC channel
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void ADC2_Init( void )
{
  DWORD regVal;

  ADC2_Stop();
  
  /* all the related pins are set to ADC inputs, AD2.7 */
  /* ADC2 channel 7 */
  SFSP0_23 = (0x00<<2)|(0x01<<0);	/* Analog input, func. 1. */

  ADC2_ACC7 = 0x0A;		/* AD2 channel 7 is 10-bit resolution */
  
  regVal = ADC2_CONFIG;
  regVal &= ~(0x01<<0);			/* single scan. */
//  regVal |= (0x01<<0);			/* continuous scan. */
  /* Positive and negative edge of MSCSS timer(START3). */  
  ADC2_CONFIG = regVal|(0x1<<14)|(0x1<<15);

  install_irq( 18, (void *)ADC2_Handler, HIGHEST_PRIORITY );
  EnableIntReq_IRQ( 18, ACTIVE_HIGH, HIGHEST_PRIORITY );

  /* Enable both Comp/match and EOS interrupts on ADC2 */ 
  ADC2_INT_SET_ENABLE = 0x03;
  return;
}

/*****************************************************************************
** Function name:		ADC2_Start
**
** Descriptions:		Start ADC channel
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void ADC2_Start( void )
{
  if ( ADC2_STATUS & 0x01 )
  {
	/* Stop ADC first if conversion in progress */
	ADC2_Stop();
  }	
  ADC2_CTRL = ((0x01<<0)|(0x01<<2));	/* Set both start and update bits. */
  return;
}

/*****************************************************************************
** Function name:		ADC2_Stop
**
** Descriptions:		Stop ADC channel
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void ADC2_Stop( void )
{
  /* if in continuous mode and conversion is progress,
  wait until it finishes. */
  while ( (ADC2_STATUS & 0x01)&&(ADC2_CONFIG & 0x01) );		
  ADC2_CTRL = (0x01<<1);
  return;
}

/*********************************************************************************
**                            End Of File
*********************************************************************************/
