/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2005
 *
 *    File name   : drv_hd44780_l.c
 *    Description : I/O function
 *
 *    History :
 *    1. Date        : June 13, 2005
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 29 $
**************************************************************************/
#include "LPC29xx.h"                        /* LPC29xx definitions */

#include "drv_hd44780_cnfg.h"
#include "drv_hd44780_l.h"

/*************************************************************************
 * Function Name: HD44780_IO_Init
 * Parameters: none
 * Return: none
 * Description: Init IO ports directions and level
 *
 *************************************************************************/
void HD44780_IO_Init (void)
{
  SFSP1_25 = (1<<2);
  SFSP1_26 = (1<<2);
  SFSP1_27 = (1<<2);
  SFSP1_28 = (1<<2);
  SFSP1_29 = (1<<2);
  SFSP1_30 = (1<<2);
  SFSP1_31 = (1<<2);
  
  GPIO1_DR |= 0x7fu<<25;
  // RS Output - Low
  HD44780SetRS(0);
  // E  Output - Low
  HD44780SetE(0);
#if HD4780_WR > 0
  // WR Output - Low
  HD44780SetRW(0);
#endif
}

/*************************************************************************
 * Function Name: HD44780SetRS
 * Parameters: Boolean Data
 * Return: none
 * Description: Set RS signal
 *
 *************************************************************************/
void HD44780SetRS (Boolean Data)
{
  if(Data)
  {
    GPIO1_OR |= 1<<25;
  }
  else
  {
    GPIO1_OR &= ~(1<<25);
  }
}

/*************************************************************************
 * Function Name: HD44780SetE
 * Parameters: Boolean Data
 * Return: none
 * Description: Set E signal
 *
 *************************************************************************/
void HD44780SetE (Boolean Data)
{
  if(Data)
  {
    GPIO1_OR |= 1<<27;
  }
  else
  {
    GPIO1_OR &= ~(1<<27);
  }
}

/*************************************************************************
 * Function Name: HD44780WrIO
 * Parameters: Int8U Data
 * Return: none
 * Description: Write to HD44780 I/O
 *
 *************************************************************************/
void HD44780WrIO (Int8U Data)
{
Int32U Tmp = (Int32U)Data << 28;
#if HD4780_WR > 0
  HD44780SetRW(0);
#endif
  // Set Direction
  GPIO1_DR |= 0x7fu<<25;
  // Write Data
  // Clock E
  HD44780SetE(1);

  GPIO1_OR = (GPIO1_OR & ~(0xfu<<28)) | Tmp;
  HD44780_BUS_DLY();
  HD44780SetE(0);
}

#if HD4780_WR > 0
/*************************************************************************
 * Function Name: HD44780SetRW
 * Parameters: Boolean Data
 * Return: none
 * Description: Set R/W signal
 *
 *************************************************************************/
void HD44780SetRW (Boolean Data)
{
  if(Data)
  {
    GPIO1_OR |= 1<<26;
  }
  else
  {
    GPIO1_OR &= ~(1<<26);
  }
}

/*************************************************************************
 * Function Name: HD44780RdIO
 * Parameters: none
 * Return: Int8U
 * Description: Read from HD44780 I/O
 *
 *************************************************************************/
Int8U HD44780RdIO (void)
{
Int8U Data;
  // Set Direction
  GPIO1_DR &= ~(0xf<<28);
  HD44780SetRW(1);
  // Read Data
  HD44780SetE(1);
  HD44780_BUS_DLY();
  Data = (GPIO1_PINS >> 28) & 0xF;
  HD44780SetE(0);
  // Clock E
  return Data;
}
#endif
