/*************************************************************************
 *
 *   Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2009
 *
 *    File name   : suart.h
 *    Description : software uart for LPC29xx header file 
 *
 *    History :
 *    1. Date        : 25.11.2009  
 *       Author      : Stoyan Choynev
 *       Description : initial revision
 *
 *    $Revision: 1.0 $
 **************************************************************************/
#ifndef __SUART_H
#define __SUART_H
/** include files **/

/**  definitions **/
#define FRM_ERR   1
#define RXB_NE    2
#define TXB_FULL  4
/** public data **/


/** public functions **/
/*************************************************************************
 * Function Name: SUART_Init
 * Parameters: None
 *
 * Return: None
 *
 * Description:SUART Initialization. RX and TX pin and Timer initialization
 *             Installs RX Timer interrupt and  enables it. Installs TX Timer
 *             interrupt.
 *             Clears RX and TX Buffers and init some internal variables.
 *
 *************************************************************************/
void SUART_Init(void);

/*************************************************************************
 * Function Name: SUART_GetStatus
 * Parameters: None
 *
 * Return:char - SUART Status
 *
 * Description: Returns SUART Status. bit 0 - Framming error
 *                                    bit 1 - Rx Buffer Not Empty
 *                                    bit 2 - Tx Buffer Full
 *************************************************************************/
extern char SUART_GetStatus(void);

/*************************************************************************
 * Function Name: SUART_GeChar
 * Parameters:None
 *
 * Return:char Data
 *
 * Description: Reads one byte from Rx Buffer
 *
 *************************************************************************/
extern char SUART_GetChar(void);

/*************************************************************************
 * Function Name: SUART_GeChar
 * Parameters:Data
 *
 * Return:char None
 *
 * Description: Pits one byte to Tx Buffer
 *
 *************************************************************************/
extern void SUART_PutChar(char data);

#endif /*_SUART_H*/
